@@ -27,7 +27,6 @@ import (
 
 	"github.com/gonvenience/bunt"
 	"github.com/gonvenience/text"
-	"github.com/gonvenience/wrap"
 	"github.com/gonvenience/ytbx"
 	"github.com/mitchellh/hashstructure"
 	yamlv3 "gopkg.in/yaml.v3"
@@ -40,19 +39,25 @@ type compareSettings struct {
 	NonStandardIdentifierGuessCountThreshold int
 	IgnoreOrderChanges                       bool
 	KubernetesEntityDetection                bool
+	AdditionalIdentifiers                    []string
 }
 
 type compare struct {
 	settings compareSettings
 }
 
-// ListItemIdentifierField names the field that identifies a list.
-type ListItemIdentifierField string
+// AdditionalIdentifiers specifies additional identifiers that will be
+// used as the key for matching maps from source to target.
+func AdditionalIdentifiers(fieldNames ...string) CompareOption {
+	return func(settings *compareSettings) {
+		settings.AdditionalIdentifiers = append(settings.AdditionalIdentifiers, fieldNames...)
+	}
+}
 
 // NonStandardIdentifierGuessCountThreshold specifies how many list entries are
 // needed for the guess-the-identifier function to actually consider the key
 // name. Or in short, if the lists only contain two entries each, there are more
-// possibilities to find unique enough key, which might no qualify as such.
+// possibilities to find unique enough key, which might not qualify as such.
 func NonStandardIdentifierGuessCountThreshold(nonStandardIdentifierGuessCountThreshold int) CompareOption {
 	return func(settings *compareSettings) {
 		settings.NonStandardIdentifierGuessCountThreshold = nonStandardIdentifierGuessCountThreshold
@@ -63,7 +68,6 @@ func NonStandardIdentifierGuessCountThreshold(nonStandardIdentifierGuessCountThr
 func IgnoreOrderChanges(value bool) CompareOption {
 	return func(settings *compareSettings) {
 		settings.IgnoreOrderChanges = value
-
 	}
 }
 
@@ -71,7 +75,6 @@ func IgnoreOrderChanges(value bool) CompareOption {
 func KubernetesEntityDetection(value bool) CompareOption {
 	return func(settings *compareSettings) {
 		settings.KubernetesEntityDetection = value
-
 	}
 }
 
@@ -102,7 +105,7 @@ func CompareInputFiles(from ytbx.InputFile, to ytbx.InputFile, compareOptions ..
 		for i := range from.Documents {
 			if entry := from.Documents[i]; !isEmptyDocument(entry) {
 				fromDocs = append(fromDocs, entry)
-				if name, err := fqrn(entry.Content[0]); err == nil {
+				if name, err := k8sItem.Name(entry.Content[0]); err == nil {
 					fromNames = append(fromNames, name)
 				}
 			}
@@ -111,7 +114,7 @@ func CompareInputFiles(from ytbx.InputFile, to ytbx.InputFile, compareOptions ..
 		for i := range to.Documents {
 			if entry := to.Documents[i]; !isEmptyDocument(entry) {
 				toDocs = append(toDocs, entry)
-				if name, err := fqrn(entry.Content[0]); err == nil {
+				if name, err := k8sItem.Name(entry.Content[0]); err == nil {
 					toNames = append(toNames, name)
 				}
 			}
@@ -209,6 +212,9 @@ func (compare *compare) nonNilSameKindNodes(path ytbx.Path, from *yamlv3.Node, t
 		case "!!null":
 			// Ignore different ways to define a null value
 
+		case "!!bool":
+			diffs, err = compare.boolValues(path, from, to)
+
 		default:
 			if from.Value != to.Value {
 				diffs, err = []Diff{{
@@ -247,7 +253,7 @@ func (compare *compare) documentNodes(from, to ytbx.InputFile) ([]Diff, error) {
 		for i, document := range inputFile.Documents {
 			node := document.Content[0]
 
-			name, err := fqrn(node)
+			name, err := k8sItem.Name(node)
 			if err != nil {
 				return nil, nil, err
 			}
@@ -335,8 +341,8 @@ func (compare *compare) documentNodes(from, to ytbx.InputFile) ([]Diff, error) {
 			if fromNames[i] != toNames[i] {
 				diff.Details = append(diff.Details, Detail{
 					Kind: ORDERCHANGE,
-					From: AsSequenceNode(fromNames),
-					To:   AsSequenceNode(toNames),
+					From: AsSequenceNode(fromNames...),
+					To:   AsSequenceNode(toNames...),
 				})
 				break
 			}
@@ -428,25 +434,22 @@ func (compare *compare) sequenceNodes(path ytbx.Path, from *yamlv3.Node, to *yam
 		return []Diff{}, nil
 	}
 
+	// check if a known identifier (e.g. name, or id) can be used
 	if identifier, err := compare.getIdentifierFromNamedLists(from, to); err == nil {
 		return compare.namedEntryLists(path, identifier, from, to)
 	}
 
-	if identifier := getNonStandardIdentifierFromNamedLists(from, to, compare.settings.NonStandardIdentifierGuessCountThreshold); identifier != "" {
-		d, err := compare.namedEntryLists(path, identifier, from, to)
-		if err != nil {
-			return nil, fmt.Errorf("sequenceNodes(nonstd): %w", err)
-		}
-
-		return d, nil
+	// check if there is a field in all entries that could serve as an identifier
+	if identifier := compare.getNonStandardIdentifierFromNamedLists(from, to); identifier != nil {
+		return compare.namedEntryLists(path, identifier, from, to)
 	}
 
-	if compare.settings.KubernetesEntityDetection {
-		if identifier, err := getIdentifierFromKubernetesEntityList(from, to); err == nil {
-			return compare.namedEntryLists(path, identifier, from, to)
-		}
+	// check if Kubernetes resource fields can be used to identify items
+	if identifier, err := compare.getIdentifierFromKubernetesEntityList(from, to); err == nil {
+		return compare.namedEntryLists(path, identifier, from, to)
 	}
 
+	// in any other case, compare lists as simple lists by relying on hashes
 	return compare.simpleLists(path, from, to)
 }
 
@@ -459,7 +462,7 @@ func (compare *compare) simpleLists(path ytbx.Path, from *yamlv3.Node, to *yamlv
 
 	// Special case if both lists only contain one entry, then directly compare
 	// the two entries with each other
-	if fromLength == 1 && fromLength == toLength {
+	if fromLength == 1 && toLength == 1 {
 		return compare.objects(
 			ytbx.NewPathWithIndexedListElement(path, 0),
 			followAlias(from.Content[0]),
@@ -528,20 +531,7 @@ func (compare *compare) simpleLists(path ytbx.Path, from *yamlv3.Node, to *yamlv
 	return packChangesAndAddToResult([]Diff{}, path, orderChanges, additions, removals)
 }
 
-func nameFromPath(node *yamlv3.Node, field ListItemIdentifierField) (string, error) {
-	parts := strings.SplitN(string(field), ".", 2)
-	key := parts[0]
-	val, err := getValueByKey(node, key)
-	if err != nil {
-		return "", fmt.Errorf("nameFromPath issue: %w", err)
-	}
-	if len(parts) == 1 {
-		return val.Value, nil
-	}
-	return nameFromPath(val, ListItemIdentifierField(parts[1]))
-}
-
-func (compare *compare) namedEntryLists(path ytbx.Path, identifier ListItemIdentifierField, from *yamlv3.Node, to *yamlv3.Node) ([]Diff, error) {
+func (compare *compare) namedEntryLists(path ytbx.Path, identifier listItemIdentifier, from *yamlv3.Node, to *yamlv3.Node) ([]Diff, error) {
 	removals := make([]*yamlv3.Node, 0)
 	additions := make([]*yamlv3.Node, 0)
 
@@ -555,12 +545,12 @@ func (compare *compare) namedEntryLists(path ytbx.Path, identifier ListItemIdent
 	// Find entries that are common to both lists to compare them separately, and
 	// find entries that are only in from, but not to and are therefore removed
 	for _, fromEntry := range from.Content {
-		name, err := nameFromPath(fromEntry, identifier)
+		name, err := identifier.Name(fromEntry)
 		if err != nil {
-			return nil, fmt.Errorf("nameEntryList from issue: %w", err)
+			return nil, fmt.Errorf("failed to identify name: %w", err)
 		}
 
-		if toEntry, ok := getEntryFromNamedList(to, identifier, name); ok {
+		if toEntry, err := identifier.FindNodeByName(to, name); err == nil {
 			// `from` and `to` have the same entry identified by identifier and name -> require comparison
 			diffs, err := compare.objects(
 				ytbx.NewPathWithNamedListElement(path, identifier, name),
@@ -581,12 +571,12 @@ func (compare *compare) namedEntryLists(path ytbx.Path, identifier ListItemIdent
 
 	// Find entries that are only in to, but not from and are therefore added
 	for _, toEntry := range to.Content {
-		name, err := nameFromPath(toEntry, identifier)
+		name, err := identifier.Name(toEntry)
 		if err != nil {
-			return nil, fmt.Errorf("nameEntryList to issue: %w", err)
+			return nil, fmt.Errorf("failed to identify name: %w", err)
 		}
 
-		if _, ok := getEntryFromNamedList(from, identifier, name); ok {
+		if _, err := identifier.FindNodeByName(from, name); err == nil {
 			// `to` and `from` have the same entry identified by identifier and name (comparison already covered by previous range)
 			toNames = append(toNames, name)
 
@@ -620,6 +610,48 @@ func (compare *compare) nodeValues(path ytbx.Path, from *yamlv3.Node, to *yamlv3
 	return result, nil
 }
 
+func (compare *compare) boolValues(path ytbx.Path, from *yamlv3.Node, to *yamlv3.Node) ([]Diff, error) {
+	boolFrom, err := toBool(from.Value)
+	if err != nil {
+		return nil, err
+	}
+	boolTo, err := toBool(to.Value)
+	if err != nil {
+		return nil, err
+	}
+	result := make([]Diff, 0)
+	if boolFrom != boolTo {
+		result = append(result, Diff{
+			&path,
+			[]Detail{{
+				Kind: MODIFICATION,
+				From: from,
+				To:   to,
+			}},
+		})
+	}
+
+	return result, nil
+}
+
+// this uses the various values mentioned in https://yaml.org/type/bool.html
+var trueValues = [...]string{"y", "Y", "yes", "Yes", "YES", "true", "True", "TRUE", "on", "On", "ON"}
+var falseValues = [...]string{"n", "N", "no", "No", "NO", "false", "False", "FALSE", "off", "Off", "OFF"}
+
+func toBool(input string) (bool, error) {
+	for _, t := range trueValues {
+		if input == t {
+			return true, nil
+		}
+	}
+	for _, f := range falseValues {
+		if input == f {
+			return false, nil
+		}
+	}
+	return false, fmt.Errorf("not a valid boolean value: '%s'", input)
+}
+
 func (compare *compare) findOrderChangesInSimpleList(fromCommon, toCommon []*yamlv3.Node) []Detail {
 	// Try to find order changes ...
 	if len(fromCommon) == len(toCommon) {
@@ -652,7 +684,7 @@ func (compare *compare) hasEntry(list []*yamlv3.Node, searchEntry *yamlv3.Node)
 }
 
 // AsSequenceNode translates a string list into a SequenceNode
-func AsSequenceNode(list []string) *yamlv3.Node {
+func AsSequenceNode(list ...string) *yamlv3.Node {
 	result := make([]*yamlv3.Node, len(list))
 	for i, entry := range list {
 		result[i] = &yamlv3.Node{
@@ -681,8 +713,8 @@ func findOrderChangesInNamedEntryLists(fromNames, toNames []string) []Detail {
 		if idxLookupMap[name] != idx {
 			orderchanges = append(orderchanges, Detail{
 				Kind: ORDERCHANGE,
-				From: AsSequenceNode(fromNames),
-				To:   AsSequenceNode(toNames),
+				From: AsSequenceNode(fromNames...),
+				To:   AsSequenceNode(toNames...),
 			})
 			break
 		}
@@ -751,39 +783,12 @@ func findValueByKey(mappingNode *yamlv3.Node, key string) (*yamlv3.Node, bool) {
 	return nil, false
 }
 
-// getValueByKey returns the value for a given key in a provided mapping node,
-// or nil with an error if there is no such entry. This is comparable to getting
-// a value from a map with `foobar[key]`.
-func getValueByKey(mappingNode *yamlv3.Node, key string) (*yamlv3.Node, error) {
-	for i := 0; i < len(mappingNode.Content); i += 2 {
-		k, v := followAlias(mappingNode.Content[i]), followAlias(mappingNode.Content[i+1])
-		if k.Value == key {
-			return v, nil
-		}
-	}
+func (compare *compare) listItemIdentifierCandidates() []string {
+	// Set default candidates that are most widly used
+	var candidates = []string{"name", "key", "id"}
 
-	if names, err := ytbx.ListStringKeys(mappingNode); err == nil {
-		return nil, fmt.Errorf("no key '%s' found in map, available keys are: %s", key, strings.Join(names, ", "))
-	}
-
-	return nil, fmt.Errorf("no key '%s' found in map and also failed to get a list of key for this map", key)
-}
-
-// getEntryFromNamedList returns the entry that is identified by the identifier
-// key and a name, for example: `name: one` where name is the identifier key and
-// one the name. Function will return nil with bool false if there is no entry.
-func getEntryFromNamedList(sequenceNode *yamlv3.Node, identifier ListItemIdentifierField, name string) (*yamlv3.Node, bool) {
-	for _, mappingNode := range sequenceNode.Content {
-		nodeName, _ := nameFromPath(mappingNode, identifier)
-		if nodeName == name {
-			return mappingNode, true
-		}
-	}
-	return nil, false
-}
-
-func (compare *compare) listItemIdentifierCandidates() []ListItemIdentifierField {
-	var candidates = []ListItemIdentifierField{"name", "key", "id"}
+	// Add user supplied additional candidates (taking precedence over defaults)
+	candidates = append(compare.settings.AdditionalIdentifiers, candidates...)
 
 	// Add Kubernetes specific extra candidate
 	if compare.settings.KubernetesEntityDetection {
@@ -793,7 +798,7 @@ func (compare *compare) listItemIdentifierCandidates() []ListItemIdentifierField
 	return candidates
 }
 
-func (compare *compare) getIdentifierFromNamedLists(listA, listB *yamlv3.Node) (ListItemIdentifierField, error) {
+func (compare *compare) getIdentifierFromNamedLists(listA, listB *yamlv3.Node) (listItemIdentifier, error) {
 	isCandidate := func(node *yamlv3.Node) bool {
 		if node.Kind == yamlv3.ScalarNode {
 			for _, entry := range compare.listItemIdentifierCandidates() {
@@ -806,19 +811,19 @@ func (compare *compare) getIdentifierFromNamedLists(listA, listB *yamlv3.Node) (
 		return false
 	}
 
-	createKeyCountMap := func(sequenceNode *yamlv3.Node) map[ListItemIdentifierField]map[string]struct{} {
-		result := map[ListItemIdentifierField]map[string]struct{}{}
+	createKeyCountMap := func(sequenceNode *yamlv3.Node) map[string]map[string]struct{} {
+		result := map[string]map[string]struct{}{}
 		for _, entry := range sequenceNode.Content {
 			switch entry.Kind {
 			case yamlv3.MappingNode:
 				for i := 0; i < len(entry.Content); i += 2 {
 					k, v := followAlias(entry.Content[i]), followAlias(entry.Content[i+1])
 					if isCandidate(k) {
-						if _, found := result[ListItemIdentifierField(k.Value)]; !found {
-							result[ListItemIdentifierField(k.Value)] = map[string]struct{}{}
+						if _, found := result[k.Value]; !found {
+							result[k.Value] = map[string]struct{}{}
 						}
 
-						result[ListItemIdentifierField(k.Value)][v.Value] = struct{}{}
+						result[k.Value][v.Value] = struct{}{}
 					}
 				}
 			}
@@ -834,81 +839,15 @@ func (compare *compare) getIdentifierFromNamedLists(listA, listB *yamlv3.Node) (
 	for _, identifier := range compare.listItemIdentifierCandidates() {
 		if countA, okA := counterA[identifier]; okA && len(countA) == len(listA.Content) {
 			if countB, okB := counterB[identifier]; okB && len(countB) == len(listB.Content) {
-				return identifier, nil
+				return &singleField{identifier}, nil
 			}
 		}
 	}
 
-	return "", fmt.Errorf("unable to find a key that can serve as an unique identifier")
+	return nil, fmt.Errorf("unable to find a key that can serve as an unique identifier")
 }
 
-// getIdentifierFromKubernetesEntityList returns 'metadata.name' as a field identifier if the provided objects all have the key.
-func getIdentifierFromKubernetesEntityList(listA, listB *yamlv3.Node) (ListItemIdentifierField, error) {
-	key := ListItemIdentifierField("metadata.name")
-	allHaveMetadataName := func(sequenceNode *yamlv3.Node) bool {
-		numWithMetadata := 0
-		for _, entry := range sequenceNode.Content {
-			switch entry.Kind {
-			case yamlv3.MappingNode:
-				_, err := nameFromPath(entry, key)
-				if err == nil {
-					numWithMetadata++
-				}
-			}
-		}
-		return numWithMetadata == len(sequenceNode.Content)
-	}
-
-	listAHasKey := allHaveMetadataName(listA)
-	listBHasKey := allHaveMetadataName(listB)
-	if listAHasKey && listBHasKey {
-		return key, nil
-	}
-
-	return "", fmt.Errorf("not all entities appear to have %q fields", key)
-}
-
-// fqrn returns something like a fully qualified Kubernetes resource name, which contains its kind, namespace and name
-func fqrn(node *yamlv3.Node) (string, error) {
-	if node.Kind != yamlv3.MappingNode {
-		return "", fmt.Errorf("name look-up for Kubernetes resources does only work with mapping nodes")
-	}
-
-	kind, err := nameFromPath(node, "kind")
-	if err != nil {
-		return "", err
-	}
-
-	namespace, err := nameFromPath(node, "metadata.namespace")
-	if err != nil {
-		namespace = "default"
-	}
-
-	name, err := nameFromPath(node, "metadata.name")
-	if err != nil {
-		return "", err
-	}
-
-	return fmt.Sprintf("%s/%s/%s", kind, namespace, name), nil
-}
-
-// isEmptyDocument returns true in case the given YAML node is an empty document
-func isEmptyDocument(node *yamlv3.Node) bool {
-	if node.Kind != yamlv3.DocumentNode {
-		return false
-	}
-
-	switch len(node.Content) {
-	case 1:
-		// special case: content is just null (scalar)
-		return node.Content[0].Kind == yamlv3.ScalarNode &&
-			node.Content[0].Tag == "!!null"
-	}
-
-	return false
-}
-
-func getNonStandardIdentifierFromNamedLists(listA, listB *yamlv3.Node, nonStandardIdentifierGuessCountThreshold int) ListItemIdentifierField {
+func (compare *compare) getNonStandardIdentifierFromNamedLists(listA, listB *yamlv3.Node) listItemIdentifier {
 	createKeyCountMap := func(list *yamlv3.Node) map[string]int {
 		tmp := map[string]map[string]struct{}{}
 		for _, entry := range list.Content {
@@ -944,13 +883,56 @@ func getNonStandardIdentifierFromNamedLists(listA, listB *yamlv3.Node, nonStanda
 
 	for keyA, countA := range counterA {
 		if countB, ok := counterB[keyA]; ok {
-			if countA == listALength && countB == listBLength && countA > nonStandardIdentifierGuessCountThreshold {
-				return ListItemIdentifierField(keyA)
+			if countA == listALength && countB == listBLength && countA > compare.settings.NonStandardIdentifierGuessCountThreshold {
+				return &singleField{keyA}
 			}
 		}
 	}
 
-	return ""
+	return nil
+}
+
+// getIdentifierFromKubernetesEntityList returns 'metadata.name' as a field identifier if the provided objects all have the key.
+func (compare *compare) getIdentifierFromKubernetesEntityList(listA, listB *yamlv3.Node) (listItemIdentifier, error) {
+	if !compare.settings.KubernetesEntityDetection {
+		return nil, fmt.Errorf("entity detection for Kubernetes resource is not enabled")
+	}
+
+	allHaveMetadataName := func(sequenceNode *yamlv3.Node) bool {
+		var numWithMetadata int
+		for _, entry := range sequenceNode.Content {
+			switch entry.Kind {
+			case yamlv3.MappingNode:
+				if _, err := k8sItem.Name(entry); err == nil {
+					numWithMetadata++
+				}
+			}
+		}
+
+		return numWithMetadata == len(sequenceNode.Content)
+	}
+
+	if allHaveMetadataName(listA) && allHaveMetadataName(listB) {
+		return k8sItem, nil
+	}
+
+	return nil, fmt.Errorf("unable to verify list entries to be Kubernetes resources using Kubernetes fields")
+}
+
+// isEmptyDocument returns true in case the given YAML node is an empty document
+func isEmptyDocument(node *yamlv3.Node) bool {
+	if node.Kind != yamlv3.DocumentNode {
+		return false
+	}
+
+	switch len(node.Content) {
+	case 1:
+		// special case: content is just null (scalar)
+		return node.Content[0].Kind == yamlv3.ScalarNode &&
+			node.Content[0].Tag == "!!null"
+	}
+
+	return false
 }
 
 func (compare *compare) createLookUpMap(sequenceNode *yamlv3.Node) map[uint64][]int {
@@ -1023,7 +1005,7 @@ func (compare *compare) calcNodeHash(node *yamlv3.Node) (hash uint64) {
 	}
 
 	if err != nil {
-		panic(wrap.Errorf(err, "failed to calculate hash of %#v", node.Value))
+		panic(fmt.Errorf("failed to calculate hash of %#v: %w", node.Value, err))
 	}
 
 	return hash
@@ -1143,3 +1125,13 @@ func pathToString(path *ytbx.Path, useGoPatchPaths bool, showPathRoot bool) stri
 
 	return result
 }
+
+func grab(node *yamlv3.Node, pathString string) (*yamlv3.Node, error) {
+	return ytbx.Grab(
+		&yamlv3.Node{
+			Kind:    yamlv3.DocumentNode,
+			Content: []*yamlv3.Node{node},
+		},
+		pathString,
+	)
+}
